#ifndef ArpStepper_h
#define ArpStepper_h

/*!
 * @file ArpStepper.h
 * This is part of arpschuino ArpStepper library  
 * version 0.1.0 adapted to ESP32
 */

//#define debug
#ifdef __AVR__
  #include "digitalWriteFast.h"
#endif
 
#define _7bits 127      ///< macro to indicate the resolution in bit 
#define _8bits 255      ///< macro to indicate the resolution in bit 
#define _10bits 1023    ///< macro to indicate the resolution in bit 
#define _12bits 4095    ///< macro to indicate the resolution in bit 
#define _16bits 65535   ///< macro to indicate the resolution in bit 

/**************************************************************************/
/*! 
    @brief  Class to create a stepper object
*/
/**************************************************************************/
class ArpStepper
{
  ////////////methodes //////////////////////
  public:

    ArpStepper();//Constructeur
    ArpStepper(byte dir,byte pull,uint16_t step_per_revolution=200);//Constructeur

    void attach(byte dir,byte pull, uint16_t step_per_revolution);
    void init(int16_t RPM_min=6,int16_t RPM_max=300,float revolutions=10,uint16_t resolution=255);//(RPM_min, RPM_max , revolution, resolution);
    void invert_rotation(bool inverted);
    
    int8_t perform();  
    bool continuous();
    void refresh(uint16_t val,uint16_t val_speed);
    void set_wait(uint16_t wait);

  private: 
  
    uint32_t calcul_nbr_steps(uint32_t val);  //calcul ne nombre de pas qu'il reste a faire  //passe a 32 bits
    bool set_direction();  
    bool step_plus(uint16_t val_speed);
    int8_t do_step(uint16_t val_speed); 
    uint32_t calcul_step_duration(uint16_t RPM, uint16_t step_per_revolution)const;
    void set_resolution(uint16_t resolution);
    const bool end_of_pulse();
    

      
//  private:    
  ////////attributs ////////////////////////  
    //bool m_mode;
    bool m_inverted;
    uint16_t m_step_per_revolution;//le nombre de pas par tour
    float m_revolutions;//le nombre de tours souhaite(en float pour avoir les nombres a virgule)
						//number of turns desired (in float to have decimal numbers)
    uint8_t m_impulse = 10;//durée de l'impulsion en micro secondes
    uint16_t m_wait = 10000;//duree de l'attente,  en ms, en cas de changement de direction//100//debug :uint16_t pas uint8_t !!!!!
    uint16_t m_RPM_min;//la vitesse minimale en tours/minute
    uint16_t m_RPM_max;//la vitesse maximale en tours/minute
     
    uint8_t m_dir;//les sorties a relier a la borne DIR du driver
    uint8_t m_pull;//les sorties a relier a la borne PUL du driver
  
  
    bool m_previous_sens;
    bool m_change_sens;
    int32_t m_nbr_pas;      
    int32_t m_previous_pas;  
    uint32_t m_previousMicros; 
    uint16_t m_resolution=255;
    float m_factor;
    //int32_t m_speed_factor;///////////////////////////////////////////////////
    float m_speed_factor;
    
    uint32_t m_motor_speed_min;//plus la valeur est grande plus la vitesse est lente
    uint32_t m_motor_speed_max;//plus la valeur est petite plus la vitesse est rapide

    bool m_must_refresh_pos_order;
    bool m_must_refresh_speed_order;

    uint32_t m_position_order;
    uint16_t m_speed_order;

    uint32_t m_interval;

    uint16_t m_val;
    uint16_t m_val_speed;


    static const uint8_t speed_curve [256];
};

#endif
