#include "custom_port.h"
#include <arduino.h>
#include <arpschuino32_core.h>
#include <arpschuino.h>
#include "port.h"
#include "I2C_port.h"
#include <vector>
using namespace std;


//put here the library you want to use

extern Port portA;
extern Port portB;
extern vector<Device*>I2C_port ;



//////////// port A //////////////
//4 steppers with homing

ArpStepper stepper0(Arp1,Arp0,800);//Arp1:DIR; Arp0:STP; 800 stp/revol
ArpStepper stepper1(Arp3,Arp2,800);
ArpStepper stepper2(Arp5,Arp4,800);
ArpStepper stepper3(Arp7,Arp6,800);	

int zeroSwitch0=Arp16;//No internal pullup in input Arp16 to 19. 
int zeroSwitch1=Arp17;//So you have to do it Using an external resistor.
int zeroSwitch2=Arp18;//see : https://roboticsbackend.com/arduino-input_pullup-pinmode/
int zeroSwitch3=Arp19;//you can also use the portB pins (Arp8, 9...) in that case set the port B as "inactive"

void setupA()   //this loop is executed once at startup
{   
    //(RPM_min(>0), RPM_max , revolutions, resolution(default 255))				
	stepper0.init(1, 500, 12, _8bits);
  stepper1.init(1, 500, 12, _8bits);
  stepper2.init(1, 500, 12, _8bits);
  stepper3.init(1, 500, 12, _8bits);

  pinMode(zeroSwitch0, INPUT);
  pinMode(zeroSwitch1, INPUT);
  pinMode(zeroSwitch2, INPUT);
  pinMode(zeroSwitch3, INPUT);

  stepper0.refresh(255, 100);//we define a direction of rotation (255, to go in the other direction : 0), and a speed (here 100 for a slow speed) 
  stepper1.refresh(255, 100);
  stepper2.refresh(255, 100);
  stepper3.refresh(255, 100);

  while (digitalRead(zeroSwitch0))//as long as the limit switch is not pressed ((! zeroSwitch) reverses the behavior) 
  {
    stepper0.continuous();//turn !
    delayMicroseconds(10);
  }
  while (digitalRead(zeroSwitch1))//home each stepper one by one
  {
    stepper1.continuous();
    delayMicroseconds(10);
  }
  while (digitalRead(zeroSwitch2))
  {
    stepper2.continuous();
    delayMicroseconds(10);
  }
  while (digitalRead(zeroSwitch3))
  {
    stepper3.continuous();
    delayMicroseconds(10);
  }

  portA.setNbDmxChannels(8);//the number of channels used in this port. REQUIRED !
  Serial.println(": custom, setup done");
  Serial.println("4 steppers with homing");  
}

void action_A(const std::vector<uint8_t> & DMXslice)  //this loop is executed at each signal reception
{
  stepper0.refresh(DMXslice[0], DMXslice[1]);
  stepper1.refresh(DMXslice[2], DMXslice[3]);
  stepper2.refresh(DMXslice[4], DMXslice[5]);
  stepper3.refresh(DMXslice[6], DMXslice[7]);
}

void at_each_loop_A()   //this loop is executed continuously
{
  stepper0.perform();
  stepper1.perform();
  stepper2.perform();
  stepper3.perform();
}



//////////// port B //////////////
//1 CC motor with end switchs

int motorPin1 = Arp8;
int motorPin2 = Arp9;
int endSwitch0=Arp12; 
int endSwitch1=Arp13;
 

void setupB()      //this loop is executed once at startup
{
  // Pont en H L298N
  pinMode(motorPin1, OUTPUT);
  pinMode(motorPin2, OUTPUT);
  pinMode(endSwitch0, INPUT_PULLUP);//see : https://roboticsbackend.com/arduino-input_pullup-pinmode/
  pinMode(endSwitch1, INPUT_PULLUP);   

  ledcSetup(8,25000,8);//channel, freq, Resolution,25000Hz should be unaudible
  ledcAttachPin(Arp8,8);// attach the channel to the GPIO to be controlled
  ledcSetup(9,25000,8);
  ledcAttachPin(Arp9,9); 

  portB.setNbDmxChannels(4);//2???//the number of channels used in this port. REQUIRED !
  Serial.println(": custom, setup done");
  Serial.println("1 CC motor with end switchs");
}

void action_B(const std::vector<uint8_t> & DMXslice)    //this loop is executed at each signal reception
{  
  uint8_t state = DMXslice[0];
  bool direction = (state > 127) ? 0 : 1;
  uint8_t speed = DMXslice[1];

  if (speed>0)
  {
    if (direction && digitalRead(endSwitch0)) // avant//forward
    {
      ledcWrite(8, speed); 
      ledcWrite(9, 0);
    }
    else if(not direction && digitalRead(endSwitch1))// arrière//backward
    {
      ledcWrite(8, 0);
      ledcWrite(9, speed); 
    }
    else 
    {
      ledcWrite(8, 0); 
      ledcWrite(9, 0);
    }       
  }
  else // Stop (freinage)//Break
  {
    ledcWrite(8, 0); 
    ledcWrite(9, 0);
  }       
}

void at_each_loop_B()   //this loop is executed continuously
{
   // nothing to do !....
}

//////////// port I2C //////////////


void setupI2C(uint8_t ID )   //this loop is executed once at startup
{
  setup();
    switch(ID)
    {
      case 0 : {
        I2C_port[ID]->setNbDmxChannels(8);//the number of channels used by this device. REQUIRED !
        //put your setup for ID0 device here
          break;
      }
      case 1 : {
        I2C_port[ID]->setNbDmxChannels(8);//the number of channels used by this device. REQUIRED !
        //put your setup for ID1 device here          
          break;
      } 
      case 2 : {
        I2C_port[ID]->setNbDmxChannels(8);//the number of channels used by this device. REQUIRED !
        //put your setup for ID0 device here          
          break;
      } 
      //etc, 6 devices max
    }                   
}

void actionI2C(uint8_t ID,const std::vector<uint8_t> & DMXslice)  //this loop is executed at each signal reception
{
  switch(ID)
  {
    case 0 : {//ID0 device
      
      break;
    }
    case 1 : {//ID1 device
      
      break;
    } 
    case 2 : {//ID2 device
      
      break;
    } 
  } 
}

void at_each_loop_I2C(uint8_t ID)   //this loop is executed continuously
{
  switch(ID)
  {
    case 0 : {//ID0 device
      
      break;
    }
    case 1 : {//ID1 device
      
      break;
    } 
    case 2 : {//ID2 device
      
      break;
    }
  }  
}