#include <arpschuino32_core.h>
#include <arpschuino.h>
#include <vector>
#include <ArtnetWiFi.h>
#include <Preferences.h>

#include "_wifi.h"
#include "arp_server.h"
#include "Port.h"
#include "I2C_port.h"
#include "signal.h"


using std::vector;

//#define BLINK_DEBUG

extern Port portA;  //objets port
extern Port portB;
extern vector<Device*>I2C_port ;
extern Preferences preferences;

extern DMX_port DMX1;

uint8_t signal_mode = 0;//0=DMX or Artnet in; 1=DMX IN 2=Artnet IN 3=DMX in Artnet out; 4= Artnet in DMX out

bool connected_to_my_network;
bool checkupdateFlag=false;
String new_version;
extern String update_status;

//Tasks declarations ///
void signal_reception( void * parameter );
void periodic_check( void * parameter );
#ifdef BLINK_DEBUG//////////vars to blink without delay (debug):///////////////////
  void blink_debug( void * parameter );
////////////////////////////////////////////////////////////
  const int blue_led = 2;
  const int green_led = 0;
  // unsigned long previousMillisB = 0;        // will store last time LED was updated
  // const long interval = 1000;           // interval at which to blink (milliseconds)
  int ledState = LOW;             // ledState used to set the LED 
#endif////////////////////////////////////////////////////////////////////////////////

void setup() //running on core 1
{
  //----------------------------------------------------Serial
  Serial.begin(115200);
  Serial.println();
  Serial.println("Version 1.1");
  //----------------------------------------------------SPIFFS
  init_SPIFFS();                  //load filesystem
  //----------------------------------------------------Preferences
  init_preferences();   //load values stored in EEPROM
  //----------------------------------------------------Voyant
  if(preferences.getBool("LED_builtin", true))
  {  
    pinMode(Arp_LED_BUILTIN,OUTPUT); //green LED 
  }   
  //----------------------------------------------------WIFI
  connected_to_my_network = start_w_connection();//true if connection is succesfull
  //----------------------------------------------------arpschuino
  int arpdress = Arp_read_arpdress_board(); //read arpdress board
  if(arpdress>0 && arpdress<513)            //if address is valid
  {
    record("DMX_address", arpdress);
    Serial.println("Get address from arpdress board : ");
  }
  else if(arpdress<1000)    //Arp_read_arpdress_board() return 1000 if no arpdress board
  {
    Serial.println("Invalid address from arpdress board, keep previous address :");
  }
  //----------------------------------------------------port   
  portA.init();       //must be called before portB.init()
  portB.init(); 
  //----------------------------------------------------I2C
  Device::init();  
  //----------------------------------------------------SERVER
  all_request(); //start server 
  //WiFi.printDiag(Serial) ;  
  	
/* signal reception task, called every 25ms */
  xTaskCreatePinnedToCore(
    signal_reception,   /* Task function. */
    "signal reception", /* name of task. */
    10000,              /* Stack size of task */
    NULL,               /* parameter of the task */
    3,                  /* priority of the task */
    NULL,               /* Task handle to keep track of created task */ 
    0);                 /* pin task to core 0 */

  // periodic_check task, called every 10 seconds */
    xTaskCreatePinnedToCore(
    periodic_check,     /* Task function. */
    "periodic_check", /* name of task. */
    10000,              /* Stack size of task */
    NULL,               /* parameter of the task */
    2,                  /* priority of the task */
    NULL,               /* Task handle to keep track of created task */ 
    0);                 /* pin task to core 0 */    

  #ifdef BLINK_DEBUG
    xTaskCreate(
      blink_debug, /* Task function. */
      "blink debug", /* name of task. */
      10000, /* Stack size of task */
      NULL, /* parameter of the task */
      2, /* priority of the task */
      NULL); /* Task handle to keep track of created task */ 
  #endif 

  // Serial.print("setup() running on core ");
  // Serial.println(xPortGetCoreID());    
}////////////////////////////////////////////////////////////////////////////////////////////////

void loop() //running on core 1
{
  portA.runAtEachLoop();
  portB.runAtEachLoop(); //portA and B run
  for(uint8_t i=0;i<Device::get_nb_devices();i++)
  {
    I2C_port[i]->runAtEachLoop();
  }
}////////////////////////////////////////////////////////////////////////////////////////////////

void signal_reception( void * parameter )//running on core 0
{
  // Serial.print("signal_reception() running on core ");
  // Serial.println(xPortGetCoreID());

  //uint8_t signal_mode = 0;//0=DMX or Artnet in; 1=DMX IN 2=Artnet IN 3=DMX in Artnet out; 4= Artnet in DMX out
  signal_mode = preferences.getUChar("signal_mode", 0);

  if(signal_mode!=2)  //init_DMX();
  {
    DMX1.DMX_init();
  }
  if(signal_mode!=1 && connected_to_my_network)//init_Artnet();
  {
    Artnet_init();
  }
  signal_action();  //infinite loop in selected mode. Run every 25ms
  //vTaskDelete( NULL );
}///////////////////////////////////////////////////////////////////////////////////

void periodic_check( void * parameter ) //running on core 0
{ 
    //   Serial.print("periodic_check() running on core ");
    //   Serial.println(xPortGetCoreID());  
  for(;;)/* loop forever */
  { 
    connected_to_my_network = WiFi.isConnected();
    if(connected_to_my_network)
    {      
      if(checkupdateFlag)
      {
        if(checkupdate())
        {
          checkSPIFFSupdate();
          update_status="update succes !";
          Serial.print(update_status);
          delay(2000);
          ESP.restart();
        }
        checkupdateFlag=false;
      }
    }
    else  //if not connected
    {
      connected_to_my_network = check_network(); //Try to reconnect
      if(signal_mode!=1 && connected_to_my_network)                 //restart Artnet
      {
        Artnet_init();      
      }
    }      
    vTaskDelay(10000);
  }
  //vTaskDelete( NULL );
}////////////////////////////////////////////////////////////////////////////

#ifdef BLINK_DEBUG//loop to blink without delay///////////////////////////////
  void blink_debug( void * parameter )
  { 
    for(;;)/* loop forever */
    {
      ledState = not(ledState);// if the LED is off turn it on and vice-versa
      // set the LED with the ledState of the variable:
      digitalWrite(blue_led, ledState);
      digitalWrite(green_led, ledState);
      vTaskDelay(1000);
    }
    vTaskDelete( NULL );
  }
#endif////////////////////////////////////////////////////////////////////////////
